/**
 * AcknowledgementBasicValidator.java
 *
 * The Copyright of this work is vested in the Australian Energy Market
 * Operator Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the Australian Energy Market Operator Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the
 * Australian Energy Market Operator Limited.
 *
 * Copyright Australian Energy Market Operator Limited. 2005
 *
 * Modification History
 *
 * Date Changed   Changed By  Description
 * ------------   ----------  -----------
 * 13-Apr-2005    adij        Created
 * 18-Aug-2006    Bob Jones   Defect 4006 - Make ReceiptID only mandatory for acknowledgement
 *                            status of ACCEPT or PARTIAL (i.e. not for REJECT).
 * 29-Jan-2010    W.Mills     Defect 6218 - Return true when no errors exists.
 * 14-Aug-2012    Bob Jones         7984 Java upgrade from 6 to 7.
 */

package com.capgemini.nemmco.b2bb.validators;

import java.util.Iterator;

import com.capgemini.nemmco.b2bb.Acknowledgement;
import com.capgemini.nemmco.b2bb.XMLObject;
import com.capgemini.nemmco.b2bb.exceptions.TransactionNotSetException;
import com.capgemini.nemmco.b2bb.utils.EventObject;
import com.capgemini.nemmco.b2bb.enums.Status; // Bob Jones 4006

public abstract class AcknowledgementBasicValidator extends XMLObjectValidator {
  protected Acknowledgement ack;
  
  /**
   * Constructor. Sets a transaction with the objects initialisation.
   * @param ack XMLObject to use in the validator.
   */
  public AcknowledgementBasicValidator(XMLObject ack) {
    super(ack);
    this.ack = (Acknowledgement) ack;
  }

  /**
   * Validate the Transaction
   * 
   * @return boolean result of validation
   * validation is called.
   */
  public boolean validate() {
    boolean result = true;

    result &= validateReceiptID() 
        & validateReceiptDate()
        & validateStatus()
        & validateDuplicate()
        & validateEvents();

    return result;
  }

  protected boolean validateReceiptID() {
    String field = Acknowledgement.RECEIPT_ID;
    // Bob Jones 4006 18/08/2006: ReceiptID is only mandatory for Accept or Partial
    if ((ack.getStatus().equals(Status.ACCEPT.getCode())) || (ack.getStatus().equals(Status.PARTIAL.getCode()))) {    
      return validator.validateMandatory(events, category, field, ack.getReceiptID())
          && validator.validateTransactionId(events, category, field, ack.getReceiptID());
    } else {
      return validator.validateTransactionId(events, category, field, ack.getReceiptID());
    }
  }

  protected boolean validateReceiptDate() {
    String field = Acknowledgement.RECEIPT_DATE;
    if (ack.isReceiptDateSet()) {
      return validator.validateMandatory(events, category, field, ack.getReceiptDate())
          && validator.validateXMLDateTime(events, category, field, ack.getReceiptDate());
    } else {
      return validator.validateMandatory(events, category, field, ack.getReceiptDisplayDate())
          && validator.validateB2BDateTime(events, category, field, ack.getReceiptDisplayDate());
    }
  }
  
  protected boolean validateStatus() {
    String field = Acknowledgement.STATUS;
    return validator.validateMandatory(events, category, field, ack.getStatus())
        && validator.validateStatus(events, category, field, ack.getStatus());
  }
  
  protected boolean validateDuplicate() {
    String field = Acknowledgement.DUPLICATE;
    return !validator.exists(ack.getDuplicate())
        || validator.validateYesNo(events, category, field, ack.getDuplicate());
  }
  
  protected boolean validateEvents() {
    boolean results = true; // W.Mills 6218
    
    if (ack.getEvents() != null && !ack.getEvents().isEmpty()) {
      Iterator<Object> iter = ack.getEvents().iterator();
      EventObject event = null;
      EventObjectBasicValidator eobv;
      while(iter.hasNext()) {
        event = (EventObject) iter.next();
        eobv = new EventObjectBasicValidator(event);
        results &= eobv.validate();
        if (eobv.getEvents() != null) {
          events.addAll(eobv.getEvents());
        }
      }
    }
    
    return results;
  }
}

