/*
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/validators/CustDetailNotificationBasicValidator.java-arc   1.12   11 Apr 2005 11:30:26   MinhN  $
 * $Revision:   1.12  $
 * $Date:   11 Apr 2005 11:30:26  $
 * 
 * Copyright 2005 Capgemini Australia Pty. Ltd. 
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 * 
 * Modification History
 * 
 * Date Changed   Changed By        Comments
 * ------------   ----------        -----------
 * 21-Oct-2005    Eddie Graham      Defect 3562 - Added validation for Initiating Transaction ID. 
 * 29 May 2007    W.Mills           d3775 Amend validate logic to avoid double message
 * 08 Jun 2007    W.Mills           d3775 Amend last modified validation to include time component
 * 21-Aug-2008    Bob Jones         Defect 4985 - increase each telephone number from one to three.
 * 09-Jun-2017       S.Premkumar    SMP - pkg7b, adding emailAddress to the contact name
 * 20-Oct-2017    H.Kaur            SMP - pkg7b, Defect 8082 fix
 * 01-Oct-2017    H.Kaur            SMP - pkg7b, Defect 8082 fix
 */

package com.capgemini.nemmco.b2bb.validators;

import com.capgemini.nemmco.b2bb.CustDetailNotification;
import com.capgemini.nemmco.b2bb.Transaction;
import com.capgemini.nemmco.b2bb.enums.ConcessionCardRebateCode;
import com.capgemini.nemmco.b2bb.enums.EventCode;
import com.capgemini.nemmco.b2bb.enums.MovementType;
import com.capgemini.nemmco.b2bb.enums.SensitiveLoad;
import com.capgemini.nemmco.b2bb.exceptions.TransactionNotSetException;
import com.capgemini.nemmco.b2bb.exceptions.UnableToLookupResourceException;
import com.capgemini.nemmco.b2bb.utils.B2BDateFormatter;

/**
 * The class <code>CustDetailNotificationBasicValidator</code> is 
 * the Basic Validator for a Customer Details Notification transaction.
 */
public class CustDetailNotificationBasicValidator extends Validator
{
  protected CustDetailNotification transaction;
  
  /**
   * Constructor
   * @param trans 
   */
  public CustDetailNotificationBasicValidator(Transaction trans)
  {
    super(trans);
    this.transaction = (CustDetailNotification) trans;
  }

  /**
   * Validate the current transaction.
   * @return boolean
   * @throws TransactionNotSetException 
   */
  public boolean validate() throws TransactionNotSetException
  {
    // d3775 Amend validate logic to avoid double message
    boolean isValid = true;
    isValid = validateTransactionID() &
           validateTransactionDate() &
           //21-Oct-2005 Eddie Graham Defect 3562 - Added validation for Initiating Transaction ID.            
           validateInitiatingTransactionID() &      
           validateNMI() &
           validateNMIChecksum() &
           validateCustomerNameOrBusinessName() &
           validateCustomerName() &
           validateBusinessName() &
           validateBusinessContactName() &           
           validateAddress() &
           validatePhone1() & 
           validatePhone2() & 
           validatePhone3() & 
           validateSensitiveLoad() &
           validateMovementType() &
           validateLastModifiedDateTime() &    
           validateConcessionCardRebateCode() &
           validateConcessionCardNumber() &
            validateEmailAddress();
     if (validateConcessionCardFromDate() & validateConcessionCardToDate()) {
        if (!validateConcessionCardDateRange())
          isValid = false;
     } else {
        isValid = false;
     }
     return isValid;
  }
  
  /**
   * 
   * @return Returns true if the initiatingTransactionID is valid
   */  
  //21-Oct-2005 Eddie Graham Defect 3562 - Added validation for Initiating Transaction ID.    
  protected boolean validateInitiatingTransactionID()
  {
    String field = Transaction.INIT_TRANSACTION_ID;
    return validator.validateTransactionId(events, category, field, transaction.getInitiatingTransactionID());
  }
  
  /**
   * Validate sensitive load
   * @return boolean
   */
  protected boolean validateSensitiveLoad()
  {
    boolean result;
    String fieldname;
    String value;
    
    fieldname = CustDetailNotification.SENSITIVE_LOAD_FIELD;
    value = transaction.getSensitiveLoad();
    result = true;
    
    result = validator.validateMandatory(events, category, fieldname, value) &&
             validator.validateEnumeratedValue(events, category, fieldname, "Senstive Load", SensitiveLoad.SENSITIVE_LOADS, value);       
    if (result && isSiteVacent() && !value.equals(SensitiveLoad.NONE.getCode()))
    {
      result = false;
      try
      {
        Object[] args = {"Sensitive Load", value};
        validator.addEvent(events, category, fieldname, args, EventCode.CODE_0202, "invalidValue");
      }
      catch (UnableToLookupResourceException e)
      {        
        validator.unexpectedException(events, fieldname,e.getMessage());
      }
    }
    return result;
  }

  /**
   * Validate movement type
   * @return boolean
   */  
  protected boolean validateMovementType()
  {
    boolean result;
    String fieldname;
    String value;
    
    fieldname = CustDetailNotification.MOVEMENT_TYPE_FIELD;
    value = transaction.getMovementType();
    result  = validator.validateMandatory(events, category, fieldname, value) &&
              validator.validateEnumeratedValue(events, category, fieldname, "Movement Type", MovementType.MOVEMENT_TYPES, value);       
    return result;    
  }

  /**
   * Validate last modified date and time
   * @return boolean
   */
  protected boolean validateLastModifiedDateTime()
  {
    boolean result;
    String dateFieldname;
    
    dateFieldname = CustDetailNotification.LAST_MODIFIED_DATE_TIME_FIELD;
    if (transaction.getDateFormat() == B2BDateFormatter.B2BDATE)
    {
      result  = validator.validateMandatory(events, category, dateFieldname, transaction.getLastModifiedDate()) &&
                validator.validateMandatory(events, category, dateFieldname, transaction.getLastModifiedTime()) &&             
                validator.validateB2BDateTime(events, category, dateFieldname, transaction.getLastModifiedDate() + " " + transaction.getLastModifiedTime()); 
    }
    else
    {
      // d3775 Amend last modified validation to include time component
      result  = validator.validateXMLDateTime(events, category, dateFieldname, transaction.getLastModifiedDateTime()); 
    }
    return result;   
  }   

  /**
   * Validate concession card rebate code
   * @return boolean
   */
  protected boolean validateConcessionCardRebateCode()
  {
    boolean result;
    String fieldname, rebateCode;
    
    fieldname = CustDetailNotification.CONCESSION_CARD_REBATE_CODE_FIELD;    
    rebateCode = transaction.getConcessionCardRebateCode();
    result = true;
    if (validator.exists(rebateCode))
    {
      result = validator.validateEnumeratedValue(events, category, fieldname, "Concession Card Rebate Code", ConcessionCardRebateCode.REBATE_CODES, rebateCode);               
    }
    return result;   
  }
  
  /**
   * Validate concession card number
   * @return boolean
   */
  protected boolean validateConcessionCardNumber()
  {
    boolean result;
    String fieldname, cardNumber;
    
    fieldname = CustDetailNotification.CONCESSION_CARD_NUMBER_FIELD;    
    cardNumber = transaction.getConcessionCardNumber();
    result = true;
    if (validator.exists(transaction.getConcessionCardRebateCode()) || 
        validator.exists(transaction.getConcessionCardFromDate()) || 
        validator.exists(transaction.getConcessionCardToDate()))
    {
      result = validator.validateMandatory(events, category, fieldname, cardNumber) &&
               validator.validateLength(events, category, fieldname, cardNumber, 10);  
    }
    else
    {
      if (validator.exists((cardNumber)))
      {
        result = validator.validateLength(events, category, fieldname, cardNumber, 10);
      }
    }
    return result;    
  }  

  /**
   * Validate concession card from date
   * @return boolean
   */  
  protected boolean validateConcessionCardFromDate()
  {
    boolean result;
    String fieldname, fromDate;
    
    fieldname = CustDetailNotification.CONCESSION_CARD_FROM_DATE_FIELD;
    fromDate = transaction.getConcessionCardFromDate();
    result = true;
    if (validator.exists(fromDate))
    {
      result = validator.validateDate(events, category, fieldname, fromDate, transaction.getDateFormat());
    }       
    return result;    
  }

  /**
   * Validate concession card to date
   * @return boolean
   */  
  protected boolean validateConcessionCardToDate()
  {
    boolean result;
    String fieldname, toDate, rebateCode;
    
    fieldname = CustDetailNotification.CONCESSION_CARD_TO_DATE_FIELD;
    toDate = transaction.getConcessionCardToDate();
    rebateCode = transaction.getConcessionCardRebateCode();
    
    result = true;    
    if (rebateCode.equals(ConcessionCardRebateCode.HEALTH_CARE_CARD.getCode()))
    {
      result = validator.validateMandatory(events, category, fieldname, toDate) &&
               validator.validateDate(events, category, fieldname, toDate, transaction.getDateFormat());          
    }
    else 
    {
      if (validator.exists(toDate))
      {
        result = validator.validateDate(events, category, fieldname, toDate, transaction.getDateFormat());
      }        
    }
    return result;    
  }

  /**
   * Validate concession card date range
   * @return boolean
   */  
  protected boolean validateConcessionCardDateRange()
  {
    boolean result;
    String fromDate, toDate, fieldname;
    
    fieldname = CustDetailNotification.CONCESSION_CARD_FROM_DATE_FIELD;
    fromDate = transaction.getConcessionCardFromDate();
    toDate = transaction.getConcessionCardToDate();
    result = true;
    if (validator.exists(fromDate) && validator.exists(toDate))
    {
      result = validator.validateDateRange(events, category, fieldname, fromDate, transaction.getDateFormat(), toDate, transaction.getDateFormat());
    }
    return result;
  }

  /**
   * Validate address
   * @return boolean
   */  
  protected boolean validateAddress()
  {
    boolean result;
    Address address;
    String fieldname = CustDetailNotification.ADDRESS_FIELD;
    
    address = transaction.getAddress();
    address.setEvents(events);    
    address.setParentCategory(category);
    address.setParentField(fieldname);
    
    result = true;
    if (isSiteVacent())
    {      
      if (address.fieldsExists())
      {
        result = address.validate();
      }
    }
    else 
    {
      result = address.validate();    
    }
    return result;
  }
 
  /**
   * Validate to ensure that only a customer name or business name is provided
   * @return boolean
   */  
  protected boolean validateCustomerNameOrBusinessName()
  {
    boolean result;
    String fieldname = CustDetailNotification.CUSTOMER_NAME_FIELD;
    PersonName customerName = transaction.getCustomerName();
    PersonName businessContactName = transaction.getBusinessContactName();
    String businessName = transaction.getBusinessName();    
    
    result = true;
    if (customerName.fieldsExists() && (validator.exists(businessName) || businessContactName.fieldsExists()))
    {
      result = false;
      try
      {
        Object[] args = {"Customer Name", "Business Name or Business Contact Name"};
        validator.addEvent(events, category, fieldname, args, EventCode.CODE_0201, "concurrentError");
      }
      catch (UnableToLookupResourceException e)
      {        
        validator.unexpectedException(events, fieldname,e.getMessage());
      }
    }
    return result;
  }

  /**
   * Validate customer name
   * @return boolean
   */
  protected boolean validateCustomerName()
  {
    boolean result;
    String fieldname = CustDetailNotification.CUSTOMER_NAME_FIELD;
    PersonName customerName = transaction.getCustomerName();
    PersonName businessContactName = transaction.getBusinessContactName();
    String businessName = transaction.getBusinessName();    
    
    customerName.setEvents(events);    
    customerName.setParentCategory(category);
    customerName.setParentField(fieldname);
    
    result = true;
    if (isSiteVacent())
    {
      if (customerName.fieldsExists() || 
          ((transaction.getAddress().fieldsExists() ||
          transaction.getPhone1().fieldsExists() ||
          transaction.getPhone2().fieldsExists() ||
          transaction.getPhone3().fieldsExists()) &&
          !validator.exists(transaction.getBusinessName())))
      {
        result = customerName.validate();
      }
    }
    else if ((!validator.exists(businessName) && !businessContactName.fieldsExists()))
    {
      result = customerName.validate();  
    } 
    return result;
  }

  /**
   * Validate business name
   * @return boolean
   */  
  protected boolean validateBusinessName()
  {
    boolean result;
    String fieldname, value;
    
    fieldname = CustDetailNotification.BUSINESS_NAME_FIELD;
    
    value = transaction.getBusinessName();
    result = true;
    if (isSiteVacent())
    {
      if (validator.exists(value) ||
          ((transaction.getAddress().fieldsExists() ||
          transaction.getPhone1().fieldsExists() ||
          transaction.getPhone2().fieldsExists() ||
          transaction.getPhone3().fieldsExists()) &&
          !transaction.getCustomerName().fieldsExists()))
      {
        result = validator.validateLengthRange(events, category, fieldname, transaction.getBusinessName(), 1, 200);    
      }
    }
    else
    {
      if (!transaction.getCustomerName().fieldsExists())
      {
        result = validator.validateMandatory(events, category, fieldname, transaction.getBusinessName()) &&
                 validator.validateLengthRange(events, category, fieldname, transaction.getBusinessName(), 1, 200);     
      }
    }     
    return result;    
  }

  /**
   * Validate business contact name
   * @return boolean
   */  
  protected boolean validateBusinessContactName()
  {
    boolean result;
    String fieldname = CustDetailNotification.BUSINESS_CONTACT_NAME_FIELD;   
    PersonName businessContactName = transaction.getBusinessContactName();
    
    businessContactName.setEvents(events);    
    businessContactName.setParentCategory(category);
    businessContactName.setParentField(fieldname);
    result = true;
    if (businessContactName.fieldsExists())
    {
      result = businessContactName.validate();            
    }
    return result;
  }

  /**
   * Validate phone number 1
   * @return boolean
   */  
  protected boolean validatePhone1()
  {
    boolean result;
    Telephone phone = transaction.getPhone1();
    
    result = true;
    phone.setEvents(events);    
    phone.setParentCategory(category);
    phone.setParentField(CustDetailNotification.PHONE1_FIELD);    
    if (phone.fieldsExists())
    {
      result = phone.validate();
    }
    return result;    
  }

  /**
   * Validate phone number 2
   * @return boolean
   */  
  protected boolean validatePhone2()
  {
    boolean result;
    Telephone phone = transaction.getPhone2();
    
    result = true;
    phone.setEvents(events);    
    phone.setParentCategory(category);
    phone.setParentField(CustDetailNotification.PHONE2_FIELD);    
    if (phone.fieldsExists())
    {
      result = phone.validate();
    }
    return result;    
  }

  /**
   * Validate phone number 3
   * @return boolean
   */  
  protected boolean validatePhone3()
  {
    boolean result;
    Telephone phone = transaction.getPhone3();
    
    result = true;
    phone.setEvents(events);    
    phone.setParentCategory(category);
    phone.setParentField(CustDetailNotification.PHONE3_FIELD);    
    if (phone.fieldsExists())
    {
      result = phone.validate();
    }
    return result;    
  }
  
  /**
   * Check if the site is currently vacant
   * @return boolean
   */  
  private boolean isSiteVacent()
  {
    return MovementType.SITE_VACANT.getCode().equals(transaction.getMovementType());
  }
  
  private boolean doesContactNameExist() {
  	return validator.exists(transaction.getCustomerName()) || validator.exists(transaction.getBusinessName());
  }
  
   /**
    * validate email address and check if person name is present if email address is present
    * @return boolean
    */
   protected boolean validateEmailAddress(){
      boolean isValid=true;
      String fieldname = CustDetailNotification.EMAIL_ADDRESS;
      String emailAddress=transaction.getEmailAddress();
      //Optional
      if (!validator.exists(emailAddress)) return isValid;

      isValid = validator.validateLengthRange(events,category, fieldname, emailAddress,0,100);
      isValid = isValid && validator.validateEmailAddress(events,category, fieldname, emailAddress);
      if(isValid && !doesContactNameExist()) {
            Object []args ={"Customer Name or Business Name","Email Address"};
            try {
               validator.addEvent(events, category, fieldname, args, EventCode.CODE_0201, "conditional");
               isValid= false;
            } catch (UnableToLookupResourceException e) {
               validator.unexpectedException(events, fieldname,e.getMessage());
               return false;
         }
      }
      return isValid;
   }

}

