/*
 * NotifiedPartyBasicValidator.java
 *
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the
 * National Electricity Market Management Company Limited.
 *
 * Copyright National Electricity Market Management Company Limited. 2009
 *
 * Modification History
 *
 * Date Changed   Changed By        Description
 * ------------   ----------        -----------
 * 23-Jan-2009    Eddie Graham      Created.
 * 18-Mar-2011    Eddie Graham      Defect 7087. Modify method validateName() to
 *	                                   a. Remove call to method validateAlphaNumeric().
 *	                                   b. Remove call to method validateLengthRange().
 *	                                   c. Add call to method validateOneWayNotificationName().
 * 17-Oct-2017    Hamish McNeish     8159 B2B Errata make NMI checksum not required
 */

package com.capgemini.nemmco.b2bb.validators;

import com.capgemini.nemmco.b2bb.NotifiedParty;
import com.capgemini.nemmco.b2bb.Transaction;
import com.capgemini.nemmco.b2bb.enums.NotificationStatusType;
import com.capgemini.nemmco.b2bb.exceptions.TransactionNotSetException;

public class NotifiedPartyBasicValidator extends Validator {
	protected NotifiedParty transaction;

	/**
	 * Constructor. Sets a transaction with the objects initialisation.
	 *
	 * @param transaction Transaction to use in the validator.
	 */
	public NotifiedPartyBasicValidator(Transaction transaction) {
		super(transaction);
		this.transaction = (NotifiedParty) transaction;
	}

	/**
	 * Validate the transaction
	 *
	 * @return boolean result of validation
	 * @throws TransactionNotSetException is a transaction was not set before
	 * validation is called.
	 */
	public boolean validate() {
		return validateTransactionID()
				   & validateTransactionDate()
				   & validateInitiatorId()
				   & validateSORecipientId()
				   & validateNMI()
				   //HAM 8159 - & validateNMIChecksum()
				   & validateServiceOrderId()
				   & validateServiceOrderType()
				   & validateServiceOrderSubType()
				   & validateScheduledDate()
				   & validateActualDateAndTime()
				   & validateNotificationStatus()
				   & validateRefTransaction();
	}

	protected boolean validateInitiatorId() {
		String fieldname = NotifiedParty.INITIATOR_ID;
		String value = transaction.getInitiatorID();
		return validator.validateMandatory(events, category, fieldname, value)
				&& validator.validateLengthRange(events, category, fieldname, value, 1, 10);
	}
	
	protected boolean validateSORecipientId() {
		String fieldname = NotifiedParty.RECIPIENT_ID;
		String value = transaction.getRecipientID();
		return validator.validateMandatory(events, category, fieldname, value)
				&& validator.validateLengthRange(events, category, fieldname, value, 1, 10);
	}
	
	protected boolean validateServiceOrderId() {
		String fieldname = NotifiedParty.SERVICE_ORDER_ID;
		String value = transaction.getServiceOrderID();
		return validator.validateMandatory(events, category, fieldname, value)
				&& validator.validateLengthRange(events, category, fieldname, value, 1, 15);
	}

	protected boolean validateRefTransaction() {
		String fieldname = NotifiedParty.REF_TRANSACTION;
		String value = transaction.getRefTransaction();
		if (!validator.validateMandatory(events, category, fieldname, value)) return false;
		//Field contains optional aseXML - nothing else to validate 
		return true;
	}

	protected boolean validateServiceOrderType() {
		String fieldname = NotifiedParty.SERVICE_ORDER_TYPE;
		String value = transaction.getServiceOrderType();
		if (!validator.validateMandatory(events, category, fieldname, value)) return false;
    return validator.validateServiceOrderType(events,
        category,
        fieldname,
        transaction.getServiceOrderType());
	}

	protected boolean validateServiceOrderSubType() {
		String fieldname = NotifiedParty.SERVICE_ORDER_SUB_TYPE;
		String value = transaction.getServiceOrderSubType();
		if (!validator.validateMandatory(events, category, fieldname, value)) return false;
    return validator.validateServiceOrderSubType(events,
        category,
        fieldname,
        transaction.getServiceOrderType(),
        transaction.getServiceOrderSubType());
	}

	protected boolean validateNotificationStatus() {
		String fieldname = NotifiedParty.NOTIFICATION_STATUS;
		String value = transaction.getNotificationStatus();
		if (!validator.validateMandatory(events, category, fieldname, value)) return false;
	  return validator.validateEnumeratedValue(events,category,fieldname,"Notification Status",NotificationStatusType.NOTIFICATION_STATUS,value);
	}

  /**
  *
  * @return Returns true if actualDateTime is valid
  */
 protected boolean validateActualDateAndTime() {
   String field = NotifiedParty.ACTUAL_DATE_AND_TIME;

   //Optional
   if (transaction.isActualDateTimeSet()) {

		  return (!validator.exists(transaction.getActualDateAndTime()) || validator.validateXMLDateTime(events, category, field, transaction.getActualDateAndTime()));
   } else {
	   if (!validator.exists(transaction.getActualDisplayDate()) && !validator.exists(transaction.getActualDisplayTime())) {
		   return true;
	   }
		  return (!validator.exists(transaction.getActualDisplayDateTime()) || validator.validateB2BDateTime(events, category, field, transaction.getActualDisplayDateTime()));
   }
 }

 protected boolean validateScheduledDate() {
		String fieldname = NotifiedParty.SCHEDULED_DATE;
    if (transaction.getIsScheduledDateSet()) {
      String value = transaction.getScheduledDate();
      if (!validator.validateMandatory(events, category, fieldname, value)) return false;
      return (!validator.exists(value))
      		|| validator.validateXMLDate(events, category, fieldname, transaction.getScheduledDate());
    }
    else {
      String value = transaction.getScheduledDisplayDate();
      if (!validator.validateMandatory(events, category, fieldname, value)) return false;
      return (!validator.exists(value))
      		|| validator.validateB2BDate(events, category, fieldname, transaction.getScheduledDisplayDate());
    }
	}

}

